# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mslib/msui/ui/ui_mscolab_project_window.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MscolabProject(object):
    def setupUi(self, MscolabProject):
        MscolabProject.setObjectName("MscolabProject")
        MscolabProject.setWindowModality(QtCore.Qt.NonModal)
        MscolabProject.setEnabled(True)
        MscolabProject.setMinimumSize(QtCore.QSize(600, 400))
        self.centralwidget = QtWidgets.QWidget(MscolabProject)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.username = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.username.sizePolicy().hasHeightForWidth())
        self.username.setSizePolicy(sizePolicy)
        self.username.setMinimumSize(QtCore.QSize(408, 0))
        self.username.setObjectName("username")
        self.horizontalLayout.addWidget(self.username)
        self.accessLevel = QtWidgets.QComboBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accessLevel.sizePolicy().hasHeightForWidth())
        self.accessLevel.setSizePolicy(sizePolicy)
        self.accessLevel.setObjectName("accessLevel")
        self.accessLevel.addItem("")
        self.accessLevel.addItem("")
        self.accessLevel.addItem("")
        self.horizontalLayout.addWidget(self.accessLevel)
        self.add = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.add.sizePolicy().hasHeightForWidth())
        self.add.setSizePolicy(sizePolicy)
        self.add.setObjectName("add")
        self.horizontalLayout.addWidget(self.add)
        self.modify = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.modify.sizePolicy().hasHeightForWidth())
        self.modify.setSizePolicy(sizePolicy)
        self.modify.setObjectName("modify")
        self.horizontalLayout.addWidget(self.modify)
        self.delete_1 = QtWidgets.QPushButton(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.delete_1.sizePolicy().hasHeightForWidth())
        self.delete_1.setSizePolicy(sizePolicy)
        self.delete_1.setObjectName("delete_1")
        self.horizontalLayout.addWidget(self.delete_1)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.proj_info = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.proj_info.sizePolicy().hasHeightForWidth())
        self.proj_info.setSizePolicy(sizePolicy)
        self.proj_info.setText("")
        self.proj_info.setObjectName("proj_info")
        self.verticalLayout_4.addWidget(self.proj_info)
        self.user_info = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.user_info.sizePolicy().hasHeightForWidth())
        self.user_info.setSizePolicy(sizePolicy)
        self.user_info.setText("")
        self.user_info.setObjectName("user_info")
        self.verticalLayout_4.addWidget(self.user_info)
        self.verticalLayout_2.addLayout(self.verticalLayout_4)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.collaboratorsList = QtWidgets.QListWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.collaboratorsList.sizePolicy().hasHeightForWidth())
        self.collaboratorsList.setSizePolicy(sizePolicy)
        self.collaboratorsList.setMinimumSize(QtCore.QSize(256, 300))
        self.collaboratorsList.setObjectName("collaboratorsList")
        self.horizontalLayout_3.addWidget(self.collaboratorsList)
        self.messages = QtWidgets.QListWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.messages.sizePolicy().hasHeightForWidth())
        self.messages.setSizePolicy(sizePolicy)
        self.messages.setMinimumSize(QtCore.QSize(539, 300))
        self.messages.setObjectName("messages")
        self.horizontalLayout_3.addWidget(self.messages)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.messageText = QtWidgets.QPlainTextEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.messageText.sizePolicy().hasHeightForWidth())
        self.messageText.setSizePolicy(sizePolicy)
        self.messageText.setMinimumSize(QtCore.QSize(400, 50))
        self.messageText.setObjectName("messageText")
        self.horizontalLayout_2.addWidget(self.messageText)
        self.sendMessage = QtWidgets.QPushButton(self.centralwidget)
        self.sendMessage.setDefault(True)
        self.sendMessage.setObjectName("sendMessage")
        self.horizontalLayout_2.addWidget(self.sendMessage)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.changes = QtWidgets.QListWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.changes.sizePolicy().hasHeightForWidth())
        self.changes.setSizePolicy(sizePolicy)
        self.changes.setMinimumSize(QtCore.QSize(400, 280))
        self.changes.setObjectName("changes")
        self.horizontalLayout_6.addWidget(self.changes)
        self.checkout = QtWidgets.QPushButton(self.centralwidget)
        self.checkout.setAutoDefault(False)
        self.checkout.setDefault(False)
        self.checkout.setObjectName("checkout")
        self.horizontalLayout_6.addWidget(self.checkout, 0, QtCore.Qt.AlignTop)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        MscolabProject.setCentralWidget(self.centralwidget)
        self.actionCloseWindow = QtWidgets.QAction(MscolabProject)
        self.actionCloseWindow.setObjectName("actionCloseWindow")
        MscolabProject.addAction(self.actionCloseWindow)

        self.retranslateUi(MscolabProject)
        self.actionCloseWindow.triggered.connect(MscolabProject.close)
        QtCore.QMetaObject.connectSlotsByName(MscolabProject)

    def retranslateUi(self, MscolabProject):
        _translate = QtCore.QCoreApplication.translate
        MscolabProject.setWindowTitle(_translate("MscolabProject", "Mscolab Project Chat"))
        self.accessLevel.setItemText(0, _translate("MscolabProject", "collaborator"))
        self.accessLevel.setItemText(1, _translate("MscolabProject", "admin"))
        self.accessLevel.setItemText(2, _translate("MscolabProject", "viewer"))
        self.add.setText(_translate("MscolabProject", "add"))
        self.modify.setText(_translate("MscolabProject", "modify"))
        self.delete_1.setText(_translate("MscolabProject", "delete"))
        self.messageText.setPlaceholderText(_translate("MscolabProject", "Enter message here"))
        self.sendMessage.setText(_translate("MscolabProject", "send"))
        self.checkout.setText(_translate("MscolabProject", "checkout"))
        self.actionCloseWindow.setText(_translate("MscolabProject", "CloseWindow"))
        self.actionCloseWindow.setShortcut(_translate("MscolabProject", "Ctrl+W"))

